package com.schbrain.canal.client.utils;

import org.springframework.context.ApplicationContext;

import java.lang.annotation.Annotation;
import java.util.*;

/**
 * @author zhuyf
 * @since 2022/6/16
 */
public class BeanUtil {

    private static ApplicationContext applicationContext;

    public BeanUtil(ApplicationContext context) {
        applicationContext = context;
    }

    /**
     * get bean
     */
    public static <T> T getBean(Class<T> type) {
        T bean;
        try {
            bean = applicationContext.getBean(type);
        } catch (Exception e) {
            bean = null;
        }
        return bean;
    }

    /**
     * get bean list
     */
    public static <T> List<T> getBeansOfType(Class<T> type) {
        Map<String, T> beanMap;
        try {
            beanMap = applicationContext.getBeansOfType(type);
        } catch (Exception e) {
            beanMap = null;
        }
        return beanMap == null ? null : new ArrayList<>(beanMap.values());
    }

    /**
     * get with annotation
     */
    public static Map<String, Object> getBeansWithAnnotation(Class<? extends Annotation> annotationType) {
        Map<String, Object> beanMap;
        try {
            beanMap = applicationContext.getBeansWithAnnotation(annotationType);
        } catch (Exception e) {
            beanMap = null;
        }
        return beanMap;
    }

}
